==> license-checker-1.13.0-sources.jar: com/vaadin/pro/licensechecker/Util.java <==
/*
 * Copyright 2000-2024 Vaadin Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.vaadin.pro.licensechecker;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URL;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Objects;

public class Util {

    public static final int EOF = -1;

    public static String toString(InputStream in) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream(1000);
        copyLarge(in, out, new byte[1000]);
        return out.toString("UTF-8");
    }

    /**
     * From commons-io IOUtils. Included to avoid external dependencies.
     */
    public static long copyLarge(final InputStream input,
            final OutputStream output, final byte[] buffer) throws IOException {
        long count = 0;
        int n;
        while (EOF != (n = input.read(buffer))) {
            output.write(buffer, 0, n);
            count += n;
        }
        return count;
    }

    public static String getHash(String content, byte[] salt, Charset charset) {
        if (content == null || content.isEmpty()) {
            return "";
        }
        return bytesToHex(Util.sha256(content, salt, charset));
    }

    public static String removeTrailingSlash(String url) {
        Objects.requireNonNull(url);
        if (url.endsWith("/")) {
            url = url.substring(0, url.length() - 1);
        }
        return url;
    }

    public static String validateURL(String url) {
        URI uri;
        try {
            uri = new URL(url).toURI();
        } catch (Exception e) {
            throw new IllegalArgumentException(
                    "Invalid URL " + url + ": " + e.getMessage(), e);
        }
        String protocol = uri.getScheme();
        if (uri.getHost() == null) {
            throw new IllegalArgumentException("Invalid URL " + url);
        }
        if (!("http".equals(protocol) || "https".equals(protocol))) {
            throw new IllegalArgumentException(
                    "Invalid URL " + url + ". Protocol must be http or https");
        }
        return url;
    }

    private static String bytesToHex(byte[] hash) {
        StringBuilder result = new StringBuilder();
        for (byte hashByte : hash) {
            String hex = Integer.toHexString(0xff & hashByte);
            if (hex.length() == 1) {
                result.append('0');
            }
            result.append(hex);
        }
        return result.toString();
    }

    private static byte[] sha256(String string, byte[] salt, Charset charset) {
        return getSha256(salt).digest(string.getBytes(charset));
    }

    private static MessageDigest getSha256(byte[] salt) {
        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-256");
            assert digest.getDigestLength() == 32;
            if (salt != null && salt.length > 0) {
                digest.update(salt);
            }
            return digest;
        } catch (NoSuchAlgorithmException e) {
            throw new UnsupportedOperationException(
                    "Your Java implementation does not support SHA-256, "
                            + "even though it is required by the Java specification. "
                            + "Change to an implementation which follows the specification.",
                    e);
        }
    }

}

==> license-checker-2.1.1-sources.jar: com/vaadin/pro/licensechecker/Util.java <==
/*
 * Copyright 2000-2024 Vaadin Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.vaadin.pro.licensechecker;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URL;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Objects;

import elemental.json.Json;
import elemental.json.JsonException;
import elemental.json.JsonObject;

public class Util {

    /**
     * A {@link DateTimeFormatter} instance defining a standardized format for
     * timestamps accepted by License Server. This format is based on the
     * ISO-8601 standard and enforced to use UTC as the time zone.
     *
     * Format Details: - Pattern: "yyyy-MM-dd'T'HH:mm:ss'Z'" - Time Zone: UTC
     */
    public static final DateTimeFormatter TIMESTAMP_FORMAT = DateTimeFormatter
            .ofPattern("yyyy-MM-dd'T'HH:mm:ss'Z'").withZone(ZoneId.of("UTC"));

    public static final int EOF = -1;

    public static String toString(InputStream in) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream(1000);
        copyLarge(in, out, new byte[1000]);
        return out.toString("UTF-8");
    }

    /**
     * From commons-io IOUtils. Included to avoid external dependencies.
     */
    public static long copyLarge(final InputStream input,
            final OutputStream output, final byte[] buffer) throws IOException {
        long count = 0;
        int n;
        while (EOF != (n = input.read(buffer))) {
            output.write(buffer, 0, n);
            count += n;
        }
        return count;
    }

    public static String getHash(String content, byte[] salt, Charset charset) {
        if (content == null || content.isEmpty()) {
            return "";
        }
        return bytesToHex(Util.sha256(content, salt, charset));
    }

    public static String removeTrailingSlash(String url) {
        Objects.requireNonNull(url);
        if (url.endsWith("/")) {
            url = url.substring(0, url.length() - 1);
        }
        return url;
    }

    public static String validateURL(String url) {
        URI uri;
        try {
            uri = new URL(url).toURI();
        } catch (Exception e) {
            throw new IllegalArgumentException(
                    "Invalid URL " + url + ": " + e.getMessage(), e);
        }
        String protocol = uri.getScheme();
        if (uri.getHost() == null) {
            throw new IllegalArgumentException("Invalid URL " + url);
        }
        if (!("http".equals(protocol) || "https".equals(protocol))) {
            throw new IllegalArgumentException(
                    "Invalid URL " + url + ". Protocol must be http or https");
        }
        return url;
    }

    private static String bytesToHex(byte[] hash) {
        StringBuilder result = new StringBuilder();
        for (byte hashByte : hash) {
            String hex = Integer.toHexString(0xff & hashByte);
            if (hex.length() == 1) {
                result.append('0');
            }
            result.append(hex);
        }
        return result.toString();
    }

    private static byte[] sha256(String string, byte[] salt, Charset charset) {
        return getSha256(salt).digest(string.getBytes(charset));
    }

    private static MessageDigest getSha256(byte[] salt) {
        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-256");
            assert digest.getDigestLength() == 32;
            if (salt != null && salt.length > 0) {
                digest.update(salt);
            }
            return digest;
        } catch (NoSuchAlgorithmException e) {
            throw new UnsupportedOperationException(
                    "Your Java implementation does not support SHA-256, "
                            + "even though it is required by the Java specification. "
                            + "Change to an implementation which follows the specification.",
                    e);
        }
    }

    /**
     * Parses a JSON response from an input stream and converts it into a
     * {@link JsonObject}. Handles errors gracefully and ensures that certain
     * server errors do not propagate sensitive information to the client
     * application.
     *
     * @param status
     *            the HTTP status code of the server response
     * @param inputStream
     *            the input stream containing the server's response
     * @return a {@link JsonObject} parsed from the input stream
     * @throws IOException
     *             if an I/O error occurs or if the JSON response cannot be
     *             parsed
     */
    public static JsonObject parseJson(int status, InputStream inputStream)
            throws IOException {
        String response;
        try {
            response = Util.toString(inputStream);
        } finally {
            inputStream.close();
        }
        try {
            return Json.parse(response);
        } catch (JsonException ex) {
            if (status == 500) {
                // Do not propagate potential License Server internal errors to
                // the caller application. Exception message is safe to
                // propagate since it will not contain the whole response
                throw new IOException(
                        "Unparsable JSON response received from server: Internal Server Error",
                        ex);
            }
            throw new IOException(
                    "Unparsable JSON response received from server: "
                            + response,
                    ex);
        }
    }

}
